// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/RivetAIDA.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Particle.hh"
#include "Rivet/Tools/ParticleIdUtils.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ChargedFinalState.hh"

#include "LWH/Profile1D.h"
#include "LWH/Histogram1D.h"
#include "LWH/AIMeasurement.h"

#include <vector>

using namespace std;

namespace Rivet {

  class LHCB_2013_I1208105 : public Analysis {
  public:

    LHCB_2013_I1208105() : Analysis("LHCB_2013_I1208105")
    {  }

    void init() {
/*
      _h_chEF_minbias = bookHistogram(1, 1, 1);  
      _h_chEF_hard = bookHistogram(2, 1, 1);
      _h_chEF_diff = bookHistogram(3, 1, 1);
      _h_chEF_nondiff = bookHistogram(4, 1, 1);
      _h_totEF_minbias = bookHistogram(5, 1, 1);
      _h_totEF_hard = bookHistogram(6, 1, 1);
      _h_totEF_diff = bookHistogram(7, 1, 1); 
      _h_totEF_nondiff = bookHistogram(8, 1, 1);

*/
      _dps_chEF_minbias = bookDataPointSet("d01-x01-y01", 10, 1.9, 4.9);
      _dps_chEF_hard = bookDataPointSet("d02-x01-y01", 10, 1.9, 4.9);
      _dps_chEF_diff = bookDataPointSet("d03-x01-y01", 10, 1.9, 4.9);
      _dps_chEF_nondiff = bookDataPointSet("d04-x01-y01", 10, 1.9, 4.9);
      _dps_totEF_minbias = bookDataPointSet("d05-x01-y01", 10, 1.9, 4.9);
      _dps_totEF_hard = bookDataPointSet("d06-x01-y01", 10, 1.9, 4.9);
      _dps_totEF_diff = bookDataPointSet("d07-x01-y01", 10, 1.9, 4.9);
      _dps_totEF_nondiff = bookDataPointSet("d08-x01-y01", 10, 1.9, 4.9);

      _tp_chEF_minbias.reset(new LWH::Profile1D(10, 1.9, 4.9));
      _tp_chEF_hard.reset(new LWH::Profile1D(10, 1.9, 4.9));
      _tp_chEF_diff.reset(new LWH::Profile1D(10, 1.9, 4.9));
      _tp_chEF_nondiff.reset(new LWH::Profile1D(10, 1.9, 4.9));
      _tp_totEF_minbias.reset(new LWH::Profile1D(10, 1.9, 4.9));
      _tp_totEF_hard.reset(new LWH::Profile1D(10, 1.9, 4.9));
      _tp_totEF_diff.reset(new LWH::Profile1D(10, 1.9, 4.9));
      _tp_totEF_nondiff.reset(new LWH::Profile1D(10, 1.9, 4.9));

      _h_chN_minbias.reset(new LWH::Histogram1D(10, 1.9, 4.9));
      _h_chN_hard.reset(new LWH::Histogram1D(10, 1.9, 4.9));
      _h_chN_diff.reset(new LWH::Histogram1D(10, 1.9, 4.9));
      _h_chN_nondiff.reset(new LWH::Histogram1D(10, 1.9, 4.9));
      _h_totN_minbias.reset(new LWH::Histogram1D(10, 1.9, 4.9));
      _h_totN_hard.reset(new LWH::Histogram1D(10, 1.9, 4.9));
      _h_totN_diff.reset(new LWH::Histogram1D(10, 1.9, 4.9));
      _h_totN_nondiff.reset(new LWH::Histogram1D(10, 1.9, 4.9));

      mbTotEvents = 0.0; hdTotEvents = 0.0; dfTotEvents = 0.0; ndTotEvents = 0.0;
      mbChEvents = 0.0; hdChEvents = 0.0; dfChEvents = 0.0; ndChEvents = 0.0;

      addProjection(FinalState(1.9, 4.9), "forwardFS");
      addProjection(FinalState(-3.5,-1.5), "backwardFS");
      addProjection(ChargedFinalState(1.9, 4.9), "forwardCFS");
      addProjection(ChargedFinalState(-3.5,-1.5), "backwardCFS");
    }

    /// Perform the per-event analysis
    void analyze(const Event& event) {
      double eta = -99.0; double energy = -1.0; double pm = -1.0;
      const double weight = event.weight();

      const FinalState& ffs = applyProjection<FinalState>(event, "forwardFS");
      const FinalState& bfs = applyProjection<FinalState>(event, "backwardFS");
      const ChargedFinalState& fcfs = applyProjection<ChargedFinalState>(event, "forwardCFS");
      const ChargedFinalState& bcfs = applyProjection<ChargedFinalState>(event, "backwardCFS");
      if (fcfs.particles().empty()) return;

      bool ishardEvt = false;
      bool isdiffEvt = false;
      foreach (const Particle& p, ffs.particles()) {
        if (p.momentum().pT() > 3.0) { ishardEvt = true; break; };
      };
      isdiffEvt = bfs.particles().empty(); // diffractive or non-diffractive?
      mbTotEvents += 1.0;
      if ( isdiffEvt ) {
        dfTotEvents += 1.0;
      } else {
        ndTotEvents += 1.0;
      };
      if ( ishardEvt ) hdTotEvents += 1.0;

      foreach (const Particle& p, ffs.particles()) {
        const FourMomentum& qmom = p.momentum();
        eta = qmom.eta();
        energy = qmom.E();
        pm = qmom.p().mod();
        //if ( ( pm < 2.0 ) || ( pm > 1000.0 ) ) continue; // avoid distribution tails
        //if ( (PID::threeCharge(p) == 0) && (qmom.pT() <= 0.2) ) continue; // ECAL selection for neutrals
        _tp_totEF_minbias->fill(eta, energy, weight);
        _h_totN_minbias->fill(eta, weight);
        if ( ishardEvt ) {			
          _tp_totEF_hard->fill(eta, energy, weight);
          _h_totN_hard->fill(eta, weight);
        };
        if ( isdiffEvt ) { 
          _tp_totEF_diff->fill(eta, energy, weight);
          _h_totN_diff->fill(eta, weight);
        } else {
          _tp_totEF_nondiff->fill(eta, energy, weight);
          _h_totN_nondiff->fill(eta, weight);
        };
      };
      // repeat for charged final state particles
      ishardEvt = false; isdiffEvt = false;
      foreach (const Particle& p, fcfs.particles()) {
        if (p.momentum().pT() > 3.0) { ishardEvt = true; break; };
      };
      isdiffEvt = bcfs.particles().empty();
      mbChEvents += 1.0;
      if ( isdiffEvt ) {
        dfChEvents += 1.0; 
      } else {
        ndChEvents += 1.0;
      };
      if ( ishardEvt ) hdChEvents += 1.0;

      foreach (const Particle& p, fcfs.particles()) {
        const FourMomentum& qmom = p.momentum();
        eta = qmom.eta();
        energy = qmom.E();
        pm = qmom.p().mod();
        //if ( ( pm < 2.0 ) || ( pm > 1000.0 ) ) continue;
        _tp_chEF_minbias->fill(eta, energy, weight);
        _h_chN_minbias->fill(eta, weight);
        if (ishardEvt) {
          _tp_chEF_hard->fill(eta, energy, weight);
          _h_chN_hard->fill(eta, weight);
        };
        if (isdiffEvt) {
          _tp_chEF_diff->fill(eta, energy, weight);
          _h_chN_diff->fill(eta, weight);
        } else {
          _tp_chEF_nondiff->fill(eta, energy, weight);
          _h_chN_nondiff->fill(eta, weight);
        };
      };
    } //end of analyze


    void finalize() {
      double norm = -1.0;
      double err = -1.0;
      double val = -1.0;
      IMeasurement *mpt;
      for(int i = 0; i < _dps_totEF_minbias->size(); ++i) {

        mpt = _dps_totEF_minbias->point(i)->coordinate(1);
        norm = 1.0/0.3/mbTotEvents;
        val = _tp_totEF_minbias->binHeight(i)*norm*_h_totN_minbias->binHeight(i);
        err = _tp_totEF_minbias->binHeight(i)*norm*_h_totN_minbias->binError(i) + _tp_totEF_minbias->binError(i)*norm*_h_totN_minbias->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

        mpt = _dps_totEF_hard->point(i)->coordinate(1);
        norm = 1.0/0.3/hdTotEvents;
        val = _tp_totEF_hard->binHeight(i)*norm*_h_totN_hard->binHeight(i);
        err = _tp_totEF_hard->binHeight(i)*norm*_h_totN_hard->binError(i) + _tp_totEF_hard->binError(i)*norm*_h_totN_hard->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

        mpt = _dps_totEF_diff->point(i)->coordinate(1);
        norm = 1.0/0.3/dfTotEvents;
        val = _tp_totEF_diff->binHeight(i)*norm*_h_totN_diff->binHeight(i);
        err = _tp_totEF_diff->binHeight(i)*norm*_h_totN_diff->binError(i) + _tp_totEF_diff->binError(i)*norm*_h_totN_diff->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

        mpt = _dps_totEF_nondiff->point(i)->coordinate(1);
        norm = 1.0/0.3/ndTotEvents;
        val = _tp_totEF_nondiff->binHeight(i)*norm*_h_totN_nondiff->binHeight(i);
        err = _tp_totEF_nondiff->binHeight(i)*norm*_h_totN_nondiff->binError(i) + _tp_totEF_nondiff->binError(i)*norm*_h_totN_nondiff->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

        mpt = _dps_chEF_minbias->point(i)->coordinate(1);
        norm = 1.0/0.3/mbChEvents;
        val = _tp_chEF_minbias->binHeight(i)*norm*_h_chN_minbias->binHeight(i);
        err = _tp_chEF_minbias->binHeight(i)*norm*_h_chN_minbias->binError(i) + _tp_chEF_minbias->binError(i)*norm*_h_chN_minbias->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

        mpt = _dps_chEF_hard->point(i)->coordinate(1);
        norm = 1.0/0.3/hdChEvents;
        val = _tp_chEF_hard->binHeight(i)*norm*_h_chN_hard->binHeight(i);
        err = _tp_chEF_hard->binHeight(i)*norm*_h_chN_hard->binError(i) + _tp_chEF_hard->binError(i)*norm*_h_chN_hard->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

        mpt = _dps_chEF_diff->point(i)->coordinate(1);
        norm = 1.0/0.3/dfChEvents;
        val = _tp_chEF_diff->binHeight(i)*norm*_h_chN_diff->binHeight(i);
        err = _tp_chEF_diff->binHeight(i)*norm*_h_chN_diff->binError(i) + _tp_chEF_diff->binError(i)*norm*_h_chN_diff->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

        mpt = _dps_chEF_nondiff->point(i)->coordinate(1);
        norm = 1.0/0.3/ndChEvents;
        val = _tp_chEF_nondiff->binHeight(i)*norm*_h_chN_nondiff->binHeight(i);
        err = _tp_chEF_nondiff->binHeight(i)*norm*_h_chN_nondiff->binError(i) + _tp_chEF_nondiff->binError(i)*norm*_h_chN_nondiff->binHeight(i);
        mpt->setValue(val);
        mpt->setErrorPlus(err);
        mpt->setErrorMinus(err);

      };
    }


  private:
    // Histograms correspond to charged and total EF for each class of events: 
    //  minimum bias, hard scattering, diffractive enriched and non-diffractive enriched

    AIDA::IDataPointSet *_dps_chEF_minbias; // will be filled in finalize with 1/d_eta <N(eta)><E(eta)>
    shared_ptr<LWH::Profile1D> _tp_chEF_minbias;  // contains <E(eta)>
    shared_ptr<LWH::Histogram1D> _h_chN_minbias; // contains N(eta)
    double mbTotEvents;

    AIDA::IDataPointSet *_dps_chEF_hard;
    shared_ptr<LWH::Profile1D> _tp_chEF_hard;
    shared_ptr<LWH::Histogram1D> _h_chN_hard;
    double hdTotEvents; // number of events in each event class (!= number of particles)

    AIDA::IDataPointSet *_dps_chEF_diff;
    shared_ptr<LWH::Profile1D> _tp_chEF_diff;
    shared_ptr<LWH::Histogram1D> _h_chN_diff;
    double dfTotEvents;

    AIDA::IDataPointSet *_dps_chEF_nondiff;
    shared_ptr<LWH::Profile1D> _tp_chEF_nondiff;
    shared_ptr<LWH::Histogram1D> _h_chN_nondiff;
    double ndTotEvents;

    AIDA::IDataPointSet *_dps_totEF_minbias;
    shared_ptr<LWH::Profile1D> _tp_totEF_minbias;
    shared_ptr<LWH::Histogram1D> _h_totN_minbias;
    double mbChEvents;

    AIDA::IDataPointSet *_dps_totEF_hard;
    shared_ptr<LWH::Profile1D> _tp_totEF_hard;
    shared_ptr<LWH::Histogram1D> _h_totN_hard;
    double hdChEvents;

    AIDA::IDataPointSet *_dps_totEF_diff;
    shared_ptr<LWH::Profile1D> _tp_totEF_diff;
    shared_ptr<LWH::Histogram1D> _h_totN_diff;
    double dfChEvents;

    AIDA::IDataPointSet *_dps_totEF_nondiff;
    shared_ptr<LWH::Profile1D> _tp_totEF_nondiff;
    shared_ptr<LWH::Histogram1D> _h_totN_nondiff;
    double ndChEvents;
  };


  // Hook for the plugin system
  DECLARE_RIVET_PLUGIN(LHCB_2013_I1208105);

}
